﻿using UnityEngine;
using System.Collections.Generic;
using System.Collections;

public class AirstrikeBehaviour : CanShoot
{
    Vector2 barrelPos = new Vector2(0.07f, 0.3f);
    Vector2 barrelDir = Vector2.up;

    AudioSource audio;
    RaycastHit2D ray;

    LineRenderer aimLine;

    public void Start()
    {
        audio = gameObject.AddComponent<AudioSource>();
        audio.outputAudioMixerGroup = Global.main.SoundEffects;
        audio.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();

        aimLine = gameObject.AddComponent<LineRenderer>();
        aimLine.startColor = new Color(0f, 1f, 0f, 0.25f);
        aimLine.endColor = new Color(0f, 1f, 0f, 0.25f);
        aimLine.material = ModAPI.FindMaterial("Sprites-Default");
    }

    void Update()
    {
        var ray2 = Physics2D.Raycast(BarrelPosition, BarrelDirection, Mathf.Infinity, LayerMask.GetMask("Bounds", "Objects"));
        aimLine.SetPosition(0, BarrelPosition);
        aimLine.SetPosition(1, ray2.point);
        aimLine.startWidth = 0.01f;
        aimLine.endWidth = 0.01f;
    }

    public void Use()
    {
        Shoot();
    }

    public override void Shoot()
    {
        audio.PlayOneShot(ModResources.confirmSFX);
        ray = Physics2D.Raycast(BarrelPosition, BarrelDirection, Mathf.Infinity, LayerMask.GetMask("Bounds", "Objects"));
        GameObject marker = new GameObject();
        marker.SetLayer(9);
        if(ray.collider.gameObject.layer == 9)
        {
            marker.transform.parent = ray.collider.transform;
            marker.AddComponent<MarkerTrack>();
        }

        var timer = Random.Range(2.5f, 5f);

        Destroy(marker, timer + 0.1f);

        marker.transform.position = ray.point;
        var line = marker.AddComponent<LineRenderer>();
        line.useWorldSpace = true;
        line.material = ModAPI.FindMaterial("VeryBright");
        line.SetPosition(0, ray.point);
        line.SetPosition(1, new Vector2(ray.point.x, ray.point.y + 10f));
        var grad = new Gradient();
        grad.SetKeys(
            new GradientColorKey[]
            {
                new GradientColorKey(Color.red, 0f),
                new GradientColorKey(Color.red, 1f)
            },
            new GradientAlphaKey[]
            {
                new GradientAlphaKey(0.15f, 0f),
                new GradientAlphaKey(0f, 1f)
            });
        line.colorGradient = grad;
        line.startWidth = 0.025f;
        line.endWidth = 0.03f;

        StartCoroutine(DropBomb(marker.transform, timer));
    }

    IEnumerator DropBomb(Transform marker, float time)
    {
        yield return new WaitForSeconds(time);
        var bobm = Instantiate(ModAPI.FindSpawnable("General Purpose Bomb").Prefab, new Vector3(marker.position.x, transform.position.y + 20f), Quaternion.identity);
        bobm.transform.Rotate(0f, 0, -90f);
        bobm.GetComponent<ExplosiveBehaviour>().ImpactForceThreshold = 0f;
        bobm.GetComponent<Rigidbody2D>().AddForce(Vector2.down * 20f, ForceMode2D.Impulse);
    }

    public override Vector2 BarrelPosition { get { return transform.TransformPoint(barrelPos); } }
    public Vector2 BarrelDirection { get { return transform.TransformDirection(barrelDir); } }
}

public class MarkerTrack : MonoBehaviour
{
    LineRenderer line;

    void Start()
    {
        line = GetComponent<LineRenderer>();
    }

    void Update()
    {
        line.SetPosition(0, transform.position);
        line.SetPosition(1, new Vector3(transform.position.x, transform.position.y + 10f));
    }
}