﻿using UnityEngine;
using System.Collections;

public class HarpoonCannonBehaviour : CanShoot
{
    AudioSource audio;

    Vector2 barrelPos = new Vector2(0.5f, 0f);
    Vector2 barrelDir = Vector2.right;

    bool isLoaded = true;
    bool Reloading = false;

    public GameObject harpoon;

    DistanceJoint2D newJoint;

    public void Awake()
    {
        audio = gameObject.AddComponent<AudioSource>();
        audio.outputAudioMixerGroup = Global.main.SoundEffects;
        audio.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();

        harpoon = transform.GetChild(0).gameObject;
    }
    public void Use()
    {
        Shoot();
    }

    public override void Shoot()
    {
        if (isLoaded && !Reloading)
        {
            audio.PlayOneShot(ModResources.harpoonLaunchSFX);
            Destroy(harpoon.GetComponent<FixedJoint2D>());
            harpoon.GetComponent<Rigidbody2D>().AddForce(BarrelPosition + BarrelDirection * 25f, ForceMode2D.Impulse);

            newJoint = harpoon.AddComponent<DistanceJoint2D>();
            newJoint.anchor = new Vector2(-0.45f, 0f);
            newJoint.connectedAnchor = new Vector2(0.5f, 0f);
            newJoint.connectedBody = gameObject.GetComponent<Rigidbody2D>();
            newJoint.maxDistanceOnly = true;
            newJoint.distance = 30f;

            JointCreated(newJoint);

            isLoaded = false;
        }
        else if(!isLoaded && !Reloading)
        {
            StartCoroutine(Reload());
        }
    }

    IEnumerator Reload()
    {
        Reloading = true;

        var distance = Vector3.Distance(gameObject.transform.position, harpoon.transform.position);

        while (newJoint.distance > 0.05f)
        {
            newJoint.distance -= distance * 0.05f;
            yield return new WaitForSeconds(0.05f);
        }
        newJoint.breakForce = 0f;
        harpoon.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);
        harpoon.transform.localPosition = new Vector3(0.5f, 0.05f);
        FixedJoint2D joint = harpoon.AddComponent<FixedJoint2D>();
        joint.connectedBody = GetComponent<Rigidbody2D>();
        Reloading = false;
        isLoaded = true;
        audio.PlayOneShot(ModResources.harpoonReloadSFX);
    }

    void JointCreated(DistanceJoint2D joint)
    {
        EnergyWireBehaviour energyWireBehaviour = joint.gameObject.AddComponent<EnergyWireBehaviour>();
        energyWireBehaviour.WireColor = Color.black;
        energyWireBehaviour.WireMaterial = Resources.Load<Material>("Materials/Wire");
        energyWireBehaviour.WireWidth = 0.025f;
        energyWireBehaviour.typedJoint = joint;
    }

    public override Vector2 BarrelPosition { get { return transform.TransformPoint(barrelPos); } }
    public Vector2 BarrelDirection { get { return transform.TransformDirection(barrelDir) * transform.localScale.x; } }
}
