﻿using UnityEngine;

class MegacannonBehaviour : CanShoot
{
    public GameObject chargeOrb;
    public SpriteRenderer orbsprite;

    Vector2 barrelPos = new Vector2(3.25f, 0f);
    Vector2 barrelDir = Vector2.right;

    float t = 0;

    RaycastHit2D[] ray;

    LineRenderer line;
    AudioSource audio;
    AudioSource audio2;

    PhysicalBehaviour phys;
    float chargeheat = 0f;

    void Start()
    {
        chargeOrb = new GameObject();
        orbsprite = chargeOrb.AddComponent<SpriteRenderer>();
        orbsprite.sprite = ModResources.beamchargeSprite;
        orbsprite.material = ModAPI.FindMaterial("VeryBright");
        orbsprite.color = Color.clear;
        chargeOrb.transform.parent = transform;
        chargeOrb.transform.localPosition = barrelPos;

        phys = GetComponent<PhysicalBehaviour>();

        line = gameObject.AddComponent<LineRenderer>();
        line.material = ModAPI.FindMaterial("VeryBright");
        line.numCapVertices = 2;
        line.startColor = Color.red;
        line.endColor = Color.red;
        line.startWidth = 0f;
        line.endWidth = 0f;
        line.rendererPriority = -5;

        audio = gameObject.AddComponent<AudioSource>();
        audio.outputAudioMixerGroup = Global.main.SoundEffects;
        audio.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
        audio.volume = 0f;
        audio.clip = ModResources.beamSFX;
        audio.loop = true;
        audio.Play();

        audio2 = gameObject.AddComponent<AudioSource>();
        audio2.outputAudioMixerGroup = Global.main.SoundEffects;
        audio2.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
        audio2.volume = 0f;
        audio2.clip = ModResources.beam2SFX;
        audio2.loop = true;
        audio2.Play();
    }

    public void UseContinuous()
    {
        t += Time.deltaTime * 1.5f;
        if (t >= 1f)
        {
            chargeheat = 1f;
            Shoot();
        }
    }

    public void Update()
    {
        t = Mathf.Clamp(t, 0f, 1f);
        t -= Time.deltaTime;

        audio.pitch = t;
        audio.volume = t * 0.5f;

        chargeOrb.transform.Rotate(0f, 0f, t * 20f);
        chargeOrb.transform.localScale = (Vector2.one * 2f) * t;
        orbsprite.color = Color.Lerp(Color.clear, Color.red, t);

        chargeheat -= Time.deltaTime * 5f;
        chargeheat = Mathf.Clamp01(chargeheat);

        ray = Physics2D.RaycastAll(BarrelPosition, BarrelDirection, Mathf.Infinity);
        line.SetPosition(0, BarrelPosition);
        line.SetPosition(1, ray[ray.Length - 1].point);

        audio2.volume = chargeheat;
        line.startWidth = chargeheat * 0.75f;
        line.endWidth = chargeheat * 0.75f;
        CameraShakeBehaviour.main.Shake(chargeheat, BarrelPosition);
    }

    public override void Shoot()
    {
        audio.volume = 0.5f;
        audio2.volume = 1f;
        line.startColor = Color.red;
        line.endColor = Color.red;
        line.startWidth = 0.75f;
        line.endWidth = 0.75f;
        for (int i = 0; i < ray.Length; i++)
        {
            if (ray[i].collider.gameObject.layer == 9)
            {
                if (ray[i].collider.TryGetComponent<PhysicalBehaviour>(out var phys))
                {
                    phys.rigidbody.AddForceAtPosition((((BarrelPosition - ray[i].point).normalized * (500f * phys.rigidbody.mass)) * Time.deltaTime) * -1f, ray[i].point);
                    phys.Ignite();
                    phys.burnIntensity += 1f;
                    phys.Temperature += 5f;
                }

                if (ray[i].collider.TryGetComponent<LimbBehaviour>(out var limb))
                {
                    limb.Wince(1.5f);
                    limb.RegenerationSpeed = 0f;
                    limb.InternalTemperature += 5f;
                    limb.Slice();
                }

                if (ray[i].collider.TryGetComponent<DestroyableBehaviour>(out var des))
                    des.Break();

                if (ray[i].collider.TryGetComponent<DamagableMachineryBehaviour>(out var mac))
                    mac.Break(Vector2.zero);
            }
        }
    }

    public override Vector2 BarrelPosition { get { return transform.TransformPoint(barrelPos); } }
    public Vector2 BarrelDirection { get { return transform.TransformDirection(barrelDir) * transform.localScale.x; } }
}
